<?php

/**
 * Zend Framework (http://framework.zend.com/)
 *
 * @link      http://github.com/zendframework/CheckOut for the canonical source repository
 * @copyright Copyright (c) 2005-2015 Zend Technologies USA Inc. (http://www.zend.com)
 * @license   http://framework.zend.com/license/new-bsd New BSD License
 */

namespace CheckOut\Controller;

use Zend\Mvc\Controller\AbstractRestfulController;
use Zend\Http\Response;
use Zend\Http\Request;
use Log\Model\LogEntity;

class CheckOutController extends AbstractRestfulController {

    public function create($data) {
        $request = $this->getRequest();

        // Check if authorised
        $isAuthorised = $this->AuthorisationController()->isAuthorisationValid($request);
        if(!$isAuthorised) {
            return $this->emptyResponse($this->getResponse(), 403);
        }

        // Create mappers
        /** @var $reservationMapper \Reservation\Model\ReservationMapper */
        $reservationMapper = $this->getMapper('ReservationMapper');
        /** @var $quantityMapper \Quantity\Model\QuantityMapper */
        $quantityMapper = $this->getMapper('QuantityMapper');

        // Get token from data
        $reservationToken = $data['reservation_token'];

        // Error if reservation token isn't given
        if (!$reservationToken) {
            return $this->emptyResponse($this->getResponse(), 400);
        }

        // Get token expiry
        $reservationTokenExpiry = $reservationMapper->getReservationTokenExpiry($reservationToken);

        // If expiry is null, it means the token doesn't exist so we return a 404
        if ($reservationTokenExpiry === null) {
            return $this->emptyResponse($this->getResponse(), 404);
        }

        // Check if it's already expired
        if ($reservationTokenExpiry <= date('Y-m-d H:i:s')) {
            $this->addLog($request, '', $reservationToken, 'Token expired, clearing reservation');

            // Remove all items for previous token
            $reservationMapper->clearReservationsByToken($reservationToken, $quantityMapper);

            return $this->emptyResponse($this->getResponse(), 408);
        }

        // Perform check out
        $isCheckedOut = $reservationMapper->checkOutByToken($reservationToken, $quantityMapper);

        // If it wasn't checked out, something went wrong somewhere so just throw an internal server error
        if (!$isCheckedOut) {
            return $this->emptyResponse($this->getResponse(), 500);
        }

        $this->addLog($request, '', $reservationToken, 'Checked Out');

        return $this->emptyResponse($this->getResponse(), 200);
    }

    private function addLog(Request $request, $productNumber, $reservationToken, $logMessage) {
        // Get header
        $headers = $request->getHeaders();
        $fromHeader = $headers->get('from');

        // Get header value
        $fromHeaderValue = $fromHeader->getFieldValue();

        // Split header value
        $splitFromHeaderValue = explode('@', $fromHeaderValue);
        $user = $splitFromHeaderValue[0];
        $device = $splitFromHeaderValue[1];

        // Get IP address
        $ipAddress = $request->getServer('REMOTE_ADDR');

        // Create log entity
        $logEntity = new LogEntity();
        $logEntity->setAction($logMessage);
        $logEntity->setDatetime(date('Y-m-d H:i:s'));
        $logEntity->setDevice($device);
        $logEntity->setIpAddress($ipAddress);
        $logEntity->setProductNumber($productNumber);
        $logEntity->setReservationToken($reservationToken);
        $logEntity->setUser($user);

        // Create mapper
        $logMapper = $this->getMapper('LogMapper');

        // Save log
        $logMapper->addLog($logEntity);
    }

    private function emptyResponse(Response $response, $statusCode) {
        $response->setContent('');
        return $response->setStatusCode($statusCode);
    }

    private function getMapper($mapperName) {
        return $this->getServiceLocator()->get($mapperName);
    }

}
